/**********************************************************************

    OKI MSM58321RS Real Time Clock/Calendar emulation

    Copyright MESS Team.
    Visit http://mamedev.org for licensing and usage restrictions.

**********************************************************************
                            _____   _____
                   CS2   1 |*    \_/     | 16  Vdd
                 WRITE   2 |             | 15  XT
                  READ   3 |             | 14  _XT
                    D0   4 |   MSM58321  | 13  CS1
                    D1   5 |   RTC58321  | 12  TEST
                    D2   6 |             | 11  STOP
                    D3   7 |             | 10  _BUSY
                   GND   8 |_____________| 9   ADDRESS WRITE

**********************************************************************/

#pragma once

#ifndef __MSM58321__
#define __MSM58321__

#include "emu.h"



//**************************************************************************
//  INTERFACE CONFIGURATION MACROS
//**************************************************************************

#define MCFG_MSM58321_ADD(_tag, _clock, _config) \
	MCFG_DEVICE_ADD(_tag, MSM58321, _clock)	\
	MCFG_DEVICE_CONFIG(_config)

#define MSM58321_INTERFACE(name) \
	const msm58321_interface (name) =



//**************************************************************************
//  TYPE DEFINITIONS
//**************************************************************************

// ======================> msm58321_interface

struct msm58321_interface
{
	devcb_write_line	m_out_busy_func;
};



// ======================> msm58321_device_config

class msm58321_device_config :   public device_config,
                                 public msm58321_interface
{
    friend class msm58321_device;

    // construction/destruction
    msm58321_device_config(const machine_config &mconfig, const char *tag, const device_config *owner, UINT32 clock);

public:
    // allocators
    static device_config *static_alloc_device_config(const machine_config &mconfig, const char *tag, const device_config *owner, UINT32 clock);
    virtual device_t *alloc_device(running_machine &machine) const;

protected:
	// device_config overrides
	virtual void device_config_complete();
};



// ======================> msm58321_device

class msm58321_device :	public device_t
{
    friend class msm58321_device_config;

    // construction/destruction
    msm58321_device(running_machine &_machine, const msm58321_device_config &_config);

public:
	DECLARE_READ8_MEMBER( read );
	DECLARE_WRITE8_MEMBER( write );

	DECLARE_WRITE_LINE_MEMBER( cs1_w );
	DECLARE_WRITE_LINE_MEMBER( cs2_w );
	DECLARE_WRITE_LINE_MEMBER( read_w );
	DECLARE_WRITE_LINE_MEMBER( write_w );
	DECLARE_WRITE_LINE_MEMBER( address_write_w );
	DECLARE_WRITE_LINE_MEMBER( stop_w );
	DECLARE_WRITE_LINE_MEMBER( test_w );
	DECLARE_READ_LINE_MEMBER( busy_r );

protected:
    // device-level overrides
    virtual void device_start();
	virtual void device_timer(emu_timer &timer, device_timer_id id, int param, void *ptr);

private:
	static const device_timer_id TIMER_CLOCK = 0;
	static const device_timer_id TIMER_BUSY = 1;

	inline int read_counter(int counter);
	inline void write_counter(int counter, int value);
	inline void advance_seconds();
	inline void advance_minutes();

	devcb_resolved_write_line	m_out_busy_func;

	int m_cs1;					// chip select 1
	int m_cs2;					// chip select 2
	int m_busy;					// busy flag
	int m_read;					// read data
	int m_write;				// write data
	int m_address_write;		// write address

	UINT8 m_reg[13];			// registers
	UINT8 m_latch;				// data latch (not present in real chip)
	UINT8 m_address;			// address latch

	// timers
	emu_timer *m_clock_timer;
	emu_timer *m_busy_timer;

	const msm58321_device_config &m_config;
};


// device type definition
extern const device_type MSM58321;



#endif
